KERNEL_IMG_NAME := .vmlinux

START_SCRIPT_CAPI := $(FILESYSTEM)/etc/init.d/E41-capi
START_SCRIPT_PIGLET := $(FILESYSTEM)/etc/init.d/S11-piglet

ADD_FILE_HELPER := $(wildcard add_file_helper/*)
ADD_FILE_HELPER := $(wildcard scripts/add-file-helper/*)

.lha.options: $(LINUX_KERNEL_BUILDDIR)/System.map
	$(Q)echo "-l 0x00008000 -e 0x00008000" > $@

$(FILESYSTEM)/$(KERNEL_IMG_NAME): $(LINUX_KERNEL_BUILDDIR)/vmlinux $(FILESYSTEM)/brcm_tools
	@echo "  OBJCOPY $(notdir $<) -> $@"
	$(CROSS_COMPILE)objcopy --remove-section .note.gnu.build-id -S -O binary $< $@
	# fuer dissassemble-helper in avmterm:
	echo `nm $< | grep "kallsyms_\(names\|token\|num_syms\|addresses\)"` >> $@
	echo `file -b -e elf $<` >>$@


dtbs = $(wildcard $(LINUX_KERNEL_BUILDDIR)/arch/$(ARCH)/boot/dts/brcm/*/Fritz_Box_*.dtb)
install: install_dtbs
install_dtbs: $(dtbs)
ifeq ($(dtbs),)
	$(error No device-tree blobs found!)
endif
	@echo "  INSTALL generated device tree blobs"
	$(foreach dtb,$(dtbs),$(call cmd_install,$(dtb),$(FILESYSTEM)/avm_dts/))

kernel: bcm_kernel

export KERNEL_DIR := $(CURDIR)/$(LINUX_KERNEL_SOURCE)
SDK_DIR := $(KERNEL_DIR)/brcm_sdk
BUILD_DIR := $(SDK_DIR)
export PROFILE := 963138BGWV_FB7569

PROFILE_SUFFIX := $(firstword $(PRODUKTE) $(FIRST_PRODUCT) $(KERNEL_LAYOUT))

override PROFILE_FILE := $(SDK_DIR)/963138BGWV_$(PROFILE_SUFFIX)

include $(SDK_DIR)/make.common
export CROSS_COMPILE

$(info BCM_KF: $(BCM_KF))
export INC_BCMDRIVER_PATH
export INC_BCMLIBS_PATH
export BUILD_ETHTOOL

# gibts da einen besseren ort für?
export CXX := $(CROSS_COMPILE)g++

export INC_UTILS_PATH INC_FLASH_PATH INC_SPI_PATH

kernelvariables_extra += INC_BCMDRIVER_PATH=$(INC_BCMDRIVER_PATH:$(KERNEL_DIR)/%=%) INC_BCMLIBS_PATH=$(INC_BCMLIBS_PATH:$(KERNEL_DIR)/%=%) BUILD_ETHTOOL=$(BUILD_ETHTOOL) CXX=$(CXX) INC_UTILS_PATH=$(INC_UTILS_PATH:$(KERNEL_DIR)/%=%) INC_FLASH_PATH=$(INC_FLASH_PATH:$(KERNEL_DIR)/%=%) INC_SPI_PATH=$(INC_SPI_PATH:$(KERNEL_DIR)/%=%)
kernelvariables_extra += BRCM_BOARD=$(BRCM_BOARD)
kernelvariables_extra += REL_INC_BRCMSHARED_PUB_PATH=$(INC_BRCMSHARED_PUB_PATH:$(KERNEL_DIR)/%=%)
kernelvariables_extra += REL_INC_BRCMSHARED_PRIV_PATH=$(INC_BRCMSHARED_PRIV_PATH:$(KERNEL_DIR)/%=%)
kernelvariables_extra += REL_INC_BRCMDRIVER_PRIV_PATH=$(INC_BRCMDRIVER_PRIV_PATH:$(KERNEL_DIR)/%=%)
kernelvariables_extra += REL_INC_BRCMDRIVER_PUB_PATH=$(INC_BRCMDRIVER_PUB_PATH:$(KERNEL_DIR)/%=%)
kernelvariables_extra += REL_BRCMDRIVERS_DIR=$(BRCMDRIVERS_DIR:$(KERNEL_DIR)/%=%)
kernelvariables_extra += SHARED_DIR=$(SHARED_DIR:$(KERNEL_DIR)/%=%)
kernelvariables_extra += INC_BRCMBOARDPARMS_PATH=$(INC_BRCMBOARDPARMS_PATH:$(KERNEL_DIR)/%=%)

$(SDK_DIR)/bcmdrivers/Makefile.autogen:
	$(MAKE) -C $(SDK_DIR)/build BUILD_DIR=$(SDK_DIR) PROFILE_FILE=$(PROFILE_FILE) bcmdrivers_autogen

bcm-header-dirs := $(INC_BRCMSHARED_PUB_PATH)/$(BRCM_BOARD) \
		$(INC_BRCMSHARED_PUB_PATH)/pmc \
		$(INC_BRCMSHARED_PUB_PATH)/drv/phys \
		$(INC_BRCMSHARED_PRIV_PATH)/$(BRCM_BOARD) \
		$(INC_BRCMDRIVER_PRIV_PATH)/$(BRCM_BOARD) \
		$(INC_BRCMDRIVER_PUB_PATH)/$(BRCM_BOARD) \
		$(BRCMDRIVERS_DIR)/broadcom/char/wlcsm_ext/impl1/include

bcm_gen_kbuild: $(addsuffix /uapi/Kbuild,$(bcm-header-dirs)) $(INC_BRCMDRIVER_PUB_PATH)/$(BRCM_BOARD)/uapi/bcm/Kbuild
	@echo "header-y += bcm/" >> $(INC_BRCMDRIVER_PUB_PATH)/$(BRCM_BOARD)/uapi/Kbuild


bcm_kernel: $(SDK_DIR)/bcmdrivers/Makefile.autogen bcm_gen_kbuild
	@echo "BCMDRIVERS AUTOGEN DONE"

.PHONY: $(FILESYSTEM)/brcm_tools
$(FILESYSTEM)/brcm_tools:
	$(call cmd_install,$(PROFILE_FILE),$@/$(PROFILE))
	$(call cmd_install,$(SDK_DIR)/make.common,$@/make.common)
	$(call cmd_install,$(SDK_DIR)/version.make,$@/version.make)

install_linux_stuff: install-brcm-build-environment

.PHONY: install-brcm-build-environment
install-brcm-build-environment: $(FILESYSTEM)/build
	$(Q)echo "  INSTALL BRCM BUILD ENVIRONMENT"
	cd $(SDK_DIR); find shared -name '*.h' -type f | $(cpio) --pass-through --unconditional --make-directories --dereference --preserve-modification-time $(CURDIR)/$(FILESYSTEM)/build/brcm_sdk
	cd $(SDK_DIR); find bcmdrivers -path bcmdrivers/broadcom/net/wl -prune -o -name '*.h' -type f | $(cpio) --pass-through --unconditional --make-directories --dereference --preserve-modification-time $(CURDIR)/$(FILESYSTEM)/build/brcm_sdk
	cd $(SDK_DIR); find bcmdrivers/broadcom/net/wl -name '*.h' -type f | $(cpio) --pass-through --unconditional --make-directories --dereference --preserve-modification-time $(CURDIR)/$(FILESYSTEM)/wl_headers/


$(SDK_DIR)/%/Kbuild: force
	$(Q)mkdir -p $(dir $@)
	$(Q)echo "  GEN     $@"
	$(Q)echo 'header-y += $(notdir $(wildcard $(subst /uapi,,$(dir $@))/*.h))' > $@

.PHONY: bcmdrivers_clean
bcmdrivers_clean:
	$(MAKE) -C $(SDK_DIR)/bcmdrivers cleanlinks

.clean.files += $(SDK_DIR)/bcmdrivers/Kconfig.autogen
.clean.files += $(SDK_DIR)/bcmdrivers/Makefile.autogen
.clean.dirs += $(addsuffix /uapi,$(bcm-header-dirs))

clean: bcmdrivers_clean
