/*
   Copyright (c) 2015 Broadcom Corporation
   All Rights Reserved

    <:label-BRCM:2015:DUAL/GPL:standard
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as published by
    the Free Software Foundation (the "GPL").
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    
    A copy of the GPL is available at http://www.broadcom.com/licenses/GPLv2.php, or by
    writing to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
    
:>
*/

#include "drivers_epon_ag.h"
#include "drv_epon_epn_tx_l1s_shp_ag.h"
#define BLOCK_ADDR_COUNT_BITS 3
#define BLOCK_ADDR_COUNT (1<<BLOCK_ADDR_COUNT_BITS)

bdmf_error_t ag_drv_epn_tx_l1s_shp_config_set(uint8_t shaper_idx, uint32_t cfgshprate, uint8_t cfgshpbstsize)
{
    uint32_t reg_config=0;

#ifdef VALIDATE_PARMS
    if((shaper_idx >= BLOCK_ADDR_COUNT) ||
       (cfgshprate >= _23BITS_MAX_VAL_))
    {
        bdmf_trace("ERROR driver %s:%u| err=%s (%d)\n", __FILE__, __LINE__, bdmf_strerror(BDMF_ERR_RANGE), BDMF_ERR_RANGE);
        return BDMF_ERR_RANGE;
    }
#endif

    reg_config = RU_FIELD_SET(shaper_idx, EPN_TX_L1S_SHP, CONFIG, CFGSHPRATE, reg_config, cfgshprate);
    reg_config = RU_FIELD_SET(shaper_idx, EPN_TX_L1S_SHP, CONFIG, CFGSHPBSTSIZE, reg_config, cfgshpbstsize);

    RU_REG_WRITE(shaper_idx, EPN_TX_L1S_SHP, CONFIG, reg_config);

    return BDMF_ERR_OK;
}

bdmf_error_t ag_drv_epn_tx_l1s_shp_config_get(uint8_t shaper_idx, uint32_t *cfgshprate, uint8_t *cfgshpbstsize)
{
    uint32_t reg_config=0;

#ifdef VALIDATE_PARMS
    if(!cfgshprate || !cfgshpbstsize)
    {
        bdmf_trace("ERROR driver %s:%u| err=%s (%d)\n", __FILE__, __LINE__, bdmf_strerror(BDMF_ERR_PARM), BDMF_ERR_PARM);
        return BDMF_ERR_PARM;
    }
    if((shaper_idx >= BLOCK_ADDR_COUNT))
    {
        bdmf_trace("ERROR driver %s:%u| err=%s (%d)\n", __FILE__, __LINE__, bdmf_strerror(BDMF_ERR_RANGE), BDMF_ERR_RANGE);
        return BDMF_ERR_RANGE;
    }
#endif

    RU_REG_READ(shaper_idx, EPN_TX_L1S_SHP, CONFIG, reg_config);

    *cfgshprate = RU_FIELD_GET(shaper_idx, EPN_TX_L1S_SHP, CONFIG, CFGSHPRATE, reg_config);
    *cfgshpbstsize = RU_FIELD_GET(shaper_idx, EPN_TX_L1S_SHP, CONFIG, CFGSHPBSTSIZE, reg_config);

    return BDMF_ERR_OK;
}

bdmf_error_t ag_drv_epn_tx_l1s_shp_que_en_set(uint8_t shaper_idx, uint8_t cfgshpen)
{
    uint32_t reg_que_en=0;

#ifdef VALIDATE_PARMS
    if((shaper_idx >= BLOCK_ADDR_COUNT))
    {
        bdmf_trace("ERROR driver %s:%u| err=%s (%d)\n", __FILE__, __LINE__, bdmf_strerror(BDMF_ERR_RANGE), BDMF_ERR_RANGE);
        return BDMF_ERR_RANGE;
    }
#endif

    reg_que_en = RU_FIELD_SET(shaper_idx, EPN_TX_L1S_SHP, QUE_EN, CFGSHPEN, reg_que_en, cfgshpen);

    RU_REG_WRITE(shaper_idx, EPN_TX_L1S_SHP, QUE_EN, reg_que_en);

    return BDMF_ERR_OK;
}

bdmf_error_t ag_drv_epn_tx_l1s_shp_que_en_get(uint8_t shaper_idx, uint8_t *cfgshpen)
{
    uint32_t reg_que_en=0;

#ifdef VALIDATE_PARMS
    if(!cfgshpen)
    {
        bdmf_trace("ERROR driver %s:%u| err=%s (%d)\n", __FILE__, __LINE__, bdmf_strerror(BDMF_ERR_PARM), BDMF_ERR_PARM);
        return BDMF_ERR_PARM;
    }
    if((shaper_idx >= BLOCK_ADDR_COUNT))
    {
        bdmf_trace("ERROR driver %s:%u| err=%s (%d)\n", __FILE__, __LINE__, bdmf_strerror(BDMF_ERR_RANGE), BDMF_ERR_RANGE);
        return BDMF_ERR_RANGE;
    }
#endif

    RU_REG_READ(shaper_idx, EPN_TX_L1S_SHP, QUE_EN, reg_que_en);

    *cfgshpen = RU_FIELD_GET(shaper_idx, EPN_TX_L1S_SHP, QUE_EN, CFGSHPEN, reg_que_en);

    return BDMF_ERR_OK;
}

#ifdef USE_BDMF_SHELL
enum
{
    BDMF_config,
    BDMF_que_en,
};

typedef enum
{
    bdmf_address_config,
    bdmf_address_que_en,
}
bdmf_address;

typedef enum ShaperIdEnum
{
    DRV_EPON_SHAPER0 = 0,
    DRV_EPON_SHAPER1,
    DRV_EPON_SHAPER2,
    DRV_EPON_SHAPER3,
    DRV_EPON_SHAPER4,
    DRV_EPON_SHAPER5,
    DRV_EPON_SHAPER6,
    DRV_EPON_SHAPER7,
    DRV_EPON_SHAPER_MAX = DRV_EPON_SHAPER7
} ShaperIdEnum;

struct bdmfmon_enum_val shaper_idx_enum_table[] = {
    {"SHAPER0 ", DRV_EPON_SHAPER0 },
    {"SHAPER1 ", DRV_EPON_SHAPER1 },
    {"SHAPER2 ", DRV_EPON_SHAPER2 },
    {"SHAPER3 ", DRV_EPON_SHAPER3 },
    {"SHAPER4 ", DRV_EPON_SHAPER4 },
    {"SHAPER5 ", DRV_EPON_SHAPER5 },
    {"SHAPER6 ", DRV_EPON_SHAPER6 },
    {"SHAPER7 ", DRV_EPON_SHAPER7 },
    {NULL, 0},
};

static int bcm_epn_tx_l1s_shp_cli_set(bdmf_session_handle session, const bdmfmon_cmd_parm_t parm[], uint16_t n_parms)
{
    bdmf_error_t err;

    switch(parm[0].value.unumber)
    {
    case BDMF_config:
        err = ag_drv_epn_tx_l1s_shp_config_set(parm[1].value.unumber, parm[2].value.unumber, parm[3].value.unumber);
        break;
    case BDMF_que_en:
        err = ag_drv_epn_tx_l1s_shp_que_en_set(parm[1].value.unumber, parm[2].value.unumber);
        break;
    default:
        err = BDMF_ERR_NOT_SUPPORTED;
        break;
    }
    return err;
}

static int bcm_epn_tx_l1s_shp_cli_get(bdmf_session_handle session, const bdmfmon_cmd_parm_t parm[], uint16_t n_parms)
{
    bdmf_error_t err;

    switch(parm[0].value.unumber)
    {
    case BDMF_config:
    {
        uint32_t cfgshprate;
        uint8_t cfgshpbstsize;
        err = ag_drv_epn_tx_l1s_shp_config_get(parm[1].value.unumber, &cfgshprate, &cfgshpbstsize);
        bdmf_session_print(session, "cfgshprate = %u = 0x%x\n", cfgshprate, cfgshprate);
        bdmf_session_print(session, "cfgshpbstsize = %u = 0x%x\n", cfgshpbstsize, cfgshpbstsize);
        break;
    }
    case BDMF_que_en:
    {
        uint8_t cfgshpen;
        err = ag_drv_epn_tx_l1s_shp_que_en_get(parm[1].value.unumber, &cfgshpen);
        bdmf_session_print(session, "cfgshpen = %u = 0x%x\n", cfgshpen, cfgshpen);
        break;
    }
    default:
        err = BDMF_ERR_NOT_SUPPORTED;
        break;
    }
    return err;
}

static int bcm_epn_tx_l1s_shp_cli_test(bdmf_session_handle session, const bdmfmon_cmd_parm_t parm[], uint16_t n_parms)
{
    bdmf_test_method m = parm[0].value.unumber;
    uint8_t shaper_idx = parm[1].value.unumber;
    bdmf_error_t err = BDMF_ERR_OK;

    {
        uint32_t cfgshprate=gtmv(m, 23);
        uint8_t cfgshpbstsize=gtmv(m, 8);
        if(!err) bdmf_session_print(session, "ag_drv_epn_tx_l1s_shp_config_set(%u %u %u)\n", shaper_idx, cfgshprate, cfgshpbstsize);
        if(!err) ag_drv_epn_tx_l1s_shp_config_set(shaper_idx, cfgshprate, cfgshpbstsize);
        if(!err) ag_drv_epn_tx_l1s_shp_config_get( shaper_idx, &cfgshprate, &cfgshpbstsize);
        if(!err) bdmf_session_print(session, "ag_drv_epn_tx_l1s_shp_config_get(%u %u %u)\n", shaper_idx, cfgshprate, cfgshpbstsize);
        if(err || cfgshprate!=gtmv(m, 23) || cfgshpbstsize!=gtmv(m, 8))
            return err ? err : BDMF_ERR_IO;
    }
    {
        uint8_t cfgshpen=gtmv(m, 8);
        if(!err) bdmf_session_print(session, "ag_drv_epn_tx_l1s_shp_que_en_set(%u %u)\n", shaper_idx, cfgshpen);
        if(!err) ag_drv_epn_tx_l1s_shp_que_en_set(shaper_idx, cfgshpen);
        if(!err) ag_drv_epn_tx_l1s_shp_que_en_get( shaper_idx, &cfgshpen);
        if(!err) bdmf_session_print(session, "ag_drv_epn_tx_l1s_shp_que_en_get(%u %u)\n", shaper_idx, cfgshpen);
        if(err || cfgshpen!=gtmv(m, 8))
            return err ? err : BDMF_ERR_IO;
    }
    return err;
}

static int bcm_epn_tx_l1s_shp_cli_address(bdmf_session_handle session, const bdmfmon_cmd_parm_t parm[], uint16_t n_parms)
{
    uint32_t i;
    uint32_t j;
    uint32_t index1_start=0;
    uint32_t index1_stop;
    uint32_t index2_start=0;
    uint32_t index2_stop;
    bdmfmon_cmd_parm_t * bdmf_parm;
    const ru_reg_rec * reg;
    const ru_block_rec * blk;
    const char * enum_string = bdmfmon_enum_parm_stringval(session, 0, parm[0].value.unumber);

    if(!enum_string)
        return BDMF_ERR_INTERNAL;

    switch (parm[0].value.unumber)
    {
    case bdmf_address_config : reg = &RU_REG(EPN_TX_L1S_SHP, CONFIG); blk = &RU_BLK(EPN_TX_L1S_SHP); break;
    case bdmf_address_que_en : reg = &RU_REG(EPN_TX_L1S_SHP, QUE_EN); blk = &RU_BLK(EPN_TX_L1S_SHP); break;
    default :
        return BDMF_ERR_NOT_SUPPORTED;
        break;
    }
    if((bdmf_parm = bdmfmon_find_named_parm(session,"index1")))
    {
        index1_start = bdmf_parm->value.unumber;
        index1_stop = index1_start + 1;
    }
    else
        index1_stop = blk->addr_count;
    if((bdmf_parm = bdmfmon_find_named_parm(session,"index2")))
    {
        index2_start = bdmf_parm->value.unumber;
        index2_stop = index2_start + 1;
    }
    else
        index2_stop = reg->ram_count + 1;
    if(index1_stop > blk->addr_count)
    {
        bdmf_session_print(session, "index1 (%u) is out of range (%u).\n", index1_stop, blk->addr_count);
        return BDMF_ERR_RANGE;
    }
    if(index2_stop > (reg->ram_count + 1))
    {
        bdmf_session_print(session, "index2 (%u) is out of range (%u).\n", index2_stop, reg->ram_count + 1);
        return BDMF_ERR_RANGE;
    }
    if(reg->ram_count)
        for (i = index1_start; i < index1_stop; i++)
        {
            bdmf_session_print(session, "index1 = %u\n", i);
            for (j = index2_start; j < index2_stop; j++)
                bdmf_session_print(session, 	 "(%5u) 0x%08X\n", j, (uint32_t)((uint32_t*)(blk->addr[i] + reg->addr) + j));
        }
    else
        for (i = index1_start; i < index1_stop; i++)
            bdmf_session_print(session, "(%3u) 0x%08X\n", i, (uint32_t)((uint32_t*)(blk->addr[i] + reg->addr)));
    return 0;
}

bdmfmon_handle_t ag_drv_epn_tx_l1s_shp_cli_init(bdmfmon_handle_t driver_dir)
{
    bdmfmon_handle_t dir;

    if ((dir = bdmfmon_dir_find(driver_dir, "epn_tx_l1s_shp"))!=NULL)
        return dir;
    dir = bdmfmon_dir_add(driver_dir, "epn_tx_l1s_shp", "epn_tx_l1s_shp", BDMF_ACCESS_ADMIN, NULL);

    {
        static bdmfmon_cmd_parm_t set_config[]={
            BDMFMON_MAKE_PARM_ENUM("shaper_idx", "shaper_idx", shaper_idx_enum_table, 0),
            BDMFMON_MAKE_PARM("cfgshprate", "cfgshprate", BDMFMON_PARM_NUMBER, 0),
            BDMFMON_MAKE_PARM("cfgshpbstsize", "cfgshpbstsize", BDMFMON_PARM_NUMBER, 0),
            BDMFMON_PARM_LIST_TERMINATOR
        };
        static bdmfmon_cmd_parm_t set_que_en[]={
            BDMFMON_MAKE_PARM_ENUM("shaper_idx", "shaper_idx", shaper_idx_enum_table, 0),
            BDMFMON_MAKE_PARM("cfgshpen", "cfgshpen", BDMFMON_PARM_NUMBER, 0),
            BDMFMON_PARM_LIST_TERMINATOR
        };
        static bdmfmon_enum_val_t selector_table[] = {
            { .name="config", .val=BDMF_config, .parms=set_config },
            { .name="que_en", .val=BDMF_que_en, .parms=set_que_en },
            BDMFMON_ENUM_LAST
        };
        BDMFMON_MAKE_CMD(dir, "set", "set", bcm_epn_tx_l1s_shp_cli_set,
            BDMFMON_MAKE_PARM_SELECTOR("purpose", "purpose", selector_table, 0));
    }
    {
        static bdmfmon_cmd_parm_t set_default[]={
            BDMFMON_MAKE_PARM_ENUM("shaper_idx", "shaper_idx", shaper_idx_enum_table, 0),
            BDMFMON_PARM_LIST_TERMINATOR
        };
        static bdmfmon_enum_val_t selector_table[] = {
            { .name="config", .val=BDMF_config, .parms=set_default },
            { .name="que_en", .val=BDMF_que_en, .parms=set_default },
            BDMFMON_ENUM_LAST
        };
        BDMFMON_MAKE_CMD(dir, "get", "get", bcm_epn_tx_l1s_shp_cli_get,
            BDMFMON_MAKE_PARM_SELECTOR("purpose", "purpose", selector_table, 0));
    }
    {
        static bdmfmon_enum_val_t enum_table_test_method[] = {
            { .name="low" , .val=bdmf_test_method_low },
            { .name="mid" , .val=bdmf_test_method_mid },
            { .name="high" , .val=bdmf_test_method_high },
            BDMFMON_ENUM_LAST
        };
        BDMFMON_MAKE_CMD(dir, "test", "test", bcm_epn_tx_l1s_shp_cli_test,
            BDMFMON_MAKE_PARM_ENUM("method", "low: 0000, mid: 1000, high: 1111", enum_table_test_method, 0),
            BDMFMON_MAKE_PARM_ENUM("shaper_idx", "shaper_idx", shaper_idx_enum_table, 0));
    }
    {
        static bdmfmon_enum_val_t enum_table_address[] = {
            { .name="CONFIG" , .val=bdmf_address_config },
            { .name="QUE_EN" , .val=bdmf_address_que_en },
            BDMFMON_ENUM_LAST
        };
        BDMFMON_MAKE_CMD(dir, "address", "address", bcm_epn_tx_l1s_shp_cli_address,
            BDMFMON_MAKE_PARM_ENUM("method", "method", enum_table_address, 0),
            BDMFMON_MAKE_PARM_ENUM("index1", "shaper_idx", shaper_idx_enum_table, 0),
            BDMFMON_MAKE_PARM("index2", "onu_id/alloc_id/port_id/etc...", BDMFMON_PARM_NUMBER, BDMFMON_PARM_FLAG_OPTIONAL));
    }
    return dir;
}
#endif /* USE_BDMF_SHELL */

