/*
   Copyright (c) 2015 Broadcom Corporation
   All Rights Reserved

    <:label-BRCM:2015:DUAL/GPL:standard
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as published by
    the Free Software Foundation (the "GPL").
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    
    A copy of the GPL is available at http://www.broadcom.com/licenses/GPLv2.php, or by
    writing to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
    
:>
*/

#include "drivers_common_ag.h"
#include "rescal_ag.h"
int ag_drv_rescal_cfg_set(uint8_t cfg_wan_rescal_rstb, uint8_t cfg_wan_rescal_diag_on, uint8_t cfg_wan_rescal_pwrdn, uint16_t cfg_wan_rescal_ctrl)
{
    uint32_t reg_cfg=0;

#ifdef VALIDATE_PARMS
    if((cfg_wan_rescal_rstb >= _1BITS_MAX_VAL_) ||
       (cfg_wan_rescal_diag_on >= _1BITS_MAX_VAL_) ||
       (cfg_wan_rescal_pwrdn >= _1BITS_MAX_VAL_) ||
       (cfg_wan_rescal_ctrl >= _13BITS_MAX_VAL_))
    {
        pr_err("ERROR driver %s:%u|(%d)\n", __FILE__, __LINE__, 0);
        return 0;
    }
#endif

    reg_cfg = RU_FIELD_SET(0, RESCAL, CFG, CFG_WAN_RESCAL_RSTB, reg_cfg, cfg_wan_rescal_rstb);
    reg_cfg = RU_FIELD_SET(0, RESCAL, CFG, CFG_WAN_RESCAL_DIAG_ON, reg_cfg, cfg_wan_rescal_diag_on);
    reg_cfg = RU_FIELD_SET(0, RESCAL, CFG, CFG_WAN_RESCAL_PWRDN, reg_cfg, cfg_wan_rescal_pwrdn);
    reg_cfg = RU_FIELD_SET(0, RESCAL, CFG, CFG_WAN_RESCAL_CTRL, reg_cfg, cfg_wan_rescal_ctrl);

    RU_REG_WRITE(0, RESCAL, CFG, reg_cfg);

    return 0;
}

int ag_drv_rescal_cfg_get(uint8_t *cfg_wan_rescal_rstb, uint8_t *cfg_wan_rescal_diag_on, uint8_t *cfg_wan_rescal_pwrdn, uint16_t *cfg_wan_rescal_ctrl)
{
    uint32_t reg_cfg=0;

#ifdef VALIDATE_PARMS
    if(!cfg_wan_rescal_rstb || !cfg_wan_rescal_diag_on || !cfg_wan_rescal_pwrdn || !cfg_wan_rescal_ctrl)
    {
        pr_err("ERROR driver %s:%u|(%d)\n", __FILE__, __LINE__, 2);
        return 2;
    }
#endif

    RU_REG_READ(0, RESCAL, CFG, reg_cfg);

    *cfg_wan_rescal_rstb = RU_FIELD_GET(0, RESCAL, CFG, CFG_WAN_RESCAL_RSTB, reg_cfg);
    *cfg_wan_rescal_diag_on = RU_FIELD_GET(0, RESCAL, CFG, CFG_WAN_RESCAL_DIAG_ON, reg_cfg);
    *cfg_wan_rescal_pwrdn = RU_FIELD_GET(0, RESCAL, CFG, CFG_WAN_RESCAL_PWRDN, reg_cfg);
    *cfg_wan_rescal_ctrl = RU_FIELD_GET(0, RESCAL, CFG, CFG_WAN_RESCAL_CTRL, reg_cfg);

    return 0;
}

int ag_drv_rescal_status_0_get(rescal_status_0 *status_0)
{
    uint32_t reg_status_0=0;

#ifdef VALIDATE_PARMS
    if(!status_0)
    {
        pr_err("ERROR driver %s:%u|(%d)\n", __FILE__, __LINE__, 2);
        return 2;
    }
#endif

    RU_REG_READ(0, RESCAL, STATUS_0, reg_status_0);

    status_0->wan_rescal_done = RU_FIELD_GET(0, RESCAL, STATUS_0, WAN_RESCAL_DONE, reg_status_0);
    status_0->wan_rescal_pon = RU_FIELD_GET(0, RESCAL, STATUS_0, WAN_RESCAL_PON, reg_status_0);
    status_0->wan_rescal_prev_comp_cnt = RU_FIELD_GET(0, RESCAL, STATUS_0, WAN_RESCAL_PREV_COMP_CNT, reg_status_0);
    status_0->wan_rescal_ctrl_dfs = RU_FIELD_GET(0, RESCAL, STATUS_0, WAN_RESCAL_CTRL_DFS, reg_status_0);
    status_0->wan_rescal_state = RU_FIELD_GET(0, RESCAL, STATUS_0, WAN_RESCAL_STATE, reg_status_0);
    status_0->wan_rescal_comp = RU_FIELD_GET(0, RESCAL, STATUS_0, WAN_RESCAL_COMP, reg_status_0);
    status_0->wan_rescal_valid = RU_FIELD_GET(0, RESCAL, STATUS_0, WAN_RESCAL_VALID, reg_status_0);

    return 0;
}

int ag_drv_rescal_status_1_get(uint8_t *wan_rescal_curr_comp_cnt)
{
    uint32_t reg_status_1=0;

#ifdef VALIDATE_PARMS
    if(!wan_rescal_curr_comp_cnt)
    {
        pr_err("ERROR driver %s:%u|(%d)\n", __FILE__, __LINE__, 2);
        return 2;
    }
#endif

    RU_REG_READ(0, RESCAL, STATUS_1, reg_status_1);

    *wan_rescal_curr_comp_cnt = RU_FIELD_GET(0, RESCAL, STATUS_1, WAN_RESCAL_CURR_COMP_CNT, reg_status_1);

    return 0;
}

