/*
   Copyright (c) 2015 Broadcom Corporation
   All Rights Reserved

    <:label-BRCM:2015:DUAL/GPL:standard
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as published by
    the Free Software Foundation (the "GPL").
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    
    A copy of the GPL is available at http://www.broadcom.com/licenses/GPLv2.php, or by
    writing to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
    
:>
*/

#include "drivers_common_ag.h"
#include "pmi_ag.h"
int ag_drv_pmi_lp_0_set(uint8_t pmi_lp_en, uint8_t pmi_lp_write)
{
    uint32_t reg_lp_0=0;

#ifdef VALIDATE_PARMS
    if((pmi_lp_write >= _1BITS_MAX_VAL_) ||
       (pmi_lp_en >= _1BITS_MAX_VAL_))
    {
        pr_err("ERROR driver %s:%u|(%d)\n", __FILE__, __LINE__, 0);
        return 0;
    }
#endif

    reg_lp_0 = RU_FIELD_SET(0, PMI, LP_0, PMI_LP_WRITE, reg_lp_0, pmi_lp_write);
    reg_lp_0 = RU_FIELD_SET(0, PMI, LP_0, PMI_LP_EN, reg_lp_0, pmi_lp_en);

    RU_REG_WRITE(0, PMI, LP_0, reg_lp_0);

    return 0;
}

int ag_drv_pmi_lp_0_get(uint8_t *pmi_lp_en, uint8_t *pmi_lp_write)
{
    uint32_t reg_lp_0=0;

#ifdef VALIDATE_PARMS
    if(!pmi_lp_write || !pmi_lp_en)
    {
        pr_err("ERROR driver %s:%u|(%d)\n", __FILE__, __LINE__, 2);
        return 2;
    }
#endif

    RU_REG_READ(0, PMI, LP_0, reg_lp_0);

    *pmi_lp_write = RU_FIELD_GET(0, PMI, LP_0, PMI_LP_WRITE, reg_lp_0);
    *pmi_lp_en = RU_FIELD_GET(0, PMI, LP_0, PMI_LP_EN, reg_lp_0);

    return 0;
}

int ag_drv_pmi_lp_1_set(uint32_t pmi_lp_addr)
{
    uint32_t reg_lp_1=0;

#ifdef VALIDATE_PARMS
#endif

    reg_lp_1 = RU_FIELD_SET(0, PMI, LP_1, PMI_LP_ADDR, reg_lp_1, pmi_lp_addr);

    RU_REG_WRITE(0, PMI, LP_1, reg_lp_1);

    return 0;
}

int ag_drv_pmi_lp_1_get(uint32_t *pmi_lp_addr)
{
    uint32_t reg_lp_1=0;

#ifdef VALIDATE_PARMS
    if(!pmi_lp_addr)
    {
        pr_err("ERROR driver %s:%u|(%d)\n", __FILE__, __LINE__, 2);
        return 2;
    }
#endif

    RU_REG_READ(0, PMI, LP_1, reg_lp_1);

    *pmi_lp_addr = RU_FIELD_GET(0, PMI, LP_1, PMI_LP_ADDR, reg_lp_1);

    return 0;
}

int ag_drv_pmi_lp_2_set(uint16_t pmi_lp_wrdata, uint16_t pmi_lp_maskdata)
{
    uint32_t reg_lp_2=0;

#ifdef VALIDATE_PARMS
#endif

    reg_lp_2 = RU_FIELD_SET(0, PMI, LP_2, PMI_LP_MASKDATA, reg_lp_2, pmi_lp_maskdata);
    reg_lp_2 = RU_FIELD_SET(0, PMI, LP_2, PMI_LP_WRDATA, reg_lp_2, pmi_lp_wrdata);

    RU_REG_WRITE(0, PMI, LP_2, reg_lp_2);

    return 0;
}

int ag_drv_pmi_lp_2_get(uint16_t *pmi_lp_wrdata, uint16_t *pmi_lp_maskdata)
{
    uint32_t reg_lp_2=0;

#ifdef VALIDATE_PARMS
    if(!pmi_lp_maskdata || !pmi_lp_wrdata)
    {
        pr_err("ERROR driver %s:%u|(%d)\n", __FILE__, __LINE__, 2);
        return 2;
    }
#endif

    RU_REG_READ(0, PMI, LP_2, reg_lp_2);

    *pmi_lp_maskdata = RU_FIELD_GET(0, PMI, LP_2, PMI_LP_MASKDATA, reg_lp_2);
    *pmi_lp_wrdata = RU_FIELD_GET(0, PMI, LP_2, PMI_LP_WRDATA, reg_lp_2);

    return 0;
}

int ag_drv_pmi_lp_3_get(uint8_t *pmi_lp_err, uint8_t *pmi_lp_ack, uint16_t *pmi_lp_rddata)
{
    uint32_t reg_lp_3=0;

#ifdef VALIDATE_PARMS
    if(!pmi_lp_rddata || !pmi_lp_ack || !pmi_lp_err)
    {
        pr_err("ERROR driver %s:%u|(%d)\n", __FILE__, __LINE__, 2);
        return 2;
    }
#endif

    RU_REG_READ(0, PMI, LP_3, reg_lp_3);

    *pmi_lp_rddata = RU_FIELD_GET(0, PMI, LP_3, PMI_LP_RDDATA, reg_lp_3);
    *pmi_lp_ack = RU_FIELD_GET(0, PMI, LP_3, PMI_LP_ACK, reg_lp_3);
    *pmi_lp_err = RU_FIELD_GET(0, PMI, LP_3, PMI_LP_ERR, reg_lp_3);

    return 0;
}

