/*
   Copyright (c) 2015 Broadcom Corporation
   All Rights Reserved

    <:label-BRCM:2015:DUAL/GPL:standard
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as published by
    the Free Software Foundation (the "GPL").
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    
    A copy of the GPL is available at http://www.broadcom.com/licenses/GPLv2.php, or by
    writing to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
    
:>
*/

#include "drivers_common_ag.h"
#include "early_txen_ag.h"
bdmf_error_t ag_drv_early_txen_txen_set(const early_txen_txen *txen)
{
    uint32_t reg_cfg=0;

#ifdef VALIDATE_PARMS
    if(!txen)
    {
        bdmf_trace("ERROR driver %s:%u| err=%s (%d)\n", __FILE__, __LINE__, bdmf_strerror(BDMF_ERR_PARM), BDMF_ERR_PARM);
        return BDMF_ERR_PARM;
    }
    if((txen->cr_xgwan_top_wan_misc_early_txen_cfg_output_txen_polarity >= _1BITS_MAX_VAL_) ||
       (txen->cr_xgwan_top_wan_misc_early_txen_cfg_input_txen_polarity >= _1BITS_MAX_VAL_) ||
       (txen->cr_xgwan_top_wan_misc_early_txen_cfg_early_txen_bypass >= _1BITS_MAX_VAL_))
    {
        bdmf_trace("ERROR driver %s:%u| err=%s (%d)\n", __FILE__, __LINE__, bdmf_strerror(BDMF_ERR_RANGE), BDMF_ERR_RANGE);
        return BDMF_ERR_RANGE;
    }
#endif

    reg_cfg = RU_FIELD_SET(0, EARLY_TXEN, CFG, HOLD_TIME, reg_cfg, txen->cr_xgwan_top_wan_misc_early_txen_cfg_hold_time);
    reg_cfg = RU_FIELD_SET(0, EARLY_TXEN, CFG, SETUP_TIME, reg_cfg, txen->cr_xgwan_top_wan_misc_early_txen_cfg_setup_time);
    reg_cfg = RU_FIELD_SET(0, EARLY_TXEN, CFG, TOFF_TIME, reg_cfg, txen->cr_xgwan_top_wan_misc_early_txen_cfg_toff_time);
    reg_cfg = RU_FIELD_SET(0, EARLY_TXEN, CFG, OUTPUT_TXEN_POLARITY, reg_cfg, txen->cr_xgwan_top_wan_misc_early_txen_cfg_output_txen_polarity);
    reg_cfg = RU_FIELD_SET(0, EARLY_TXEN, CFG, INPUT_TXEN_POLARITY, reg_cfg, txen->cr_xgwan_top_wan_misc_early_txen_cfg_input_txen_polarity);
    reg_cfg = RU_FIELD_SET(0, EARLY_TXEN, CFG, EARLY_TXEN_BYPASS, reg_cfg, txen->cr_xgwan_top_wan_misc_early_txen_cfg_early_txen_bypass);

    RU_REG_WRITE(0, EARLY_TXEN, CFG, reg_cfg);

    return BDMF_ERR_OK;
}

bdmf_error_t ag_drv_early_txen_txen_get(early_txen_txen *txen)
{
    uint32_t reg_cfg=0;

#ifdef VALIDATE_PARMS
    if(!txen)
    {
        bdmf_trace("ERROR driver %s:%u| err=%s (%d)\n", __FILE__, __LINE__, bdmf_strerror(BDMF_ERR_PARM), BDMF_ERR_PARM);
        return BDMF_ERR_PARM;
    }
#endif

    RU_REG_READ(0, EARLY_TXEN, CFG, reg_cfg);

    txen->cr_xgwan_top_wan_misc_early_txen_cfg_hold_time = RU_FIELD_GET(0, EARLY_TXEN, CFG, HOLD_TIME, reg_cfg);
    txen->cr_xgwan_top_wan_misc_early_txen_cfg_setup_time = RU_FIELD_GET(0, EARLY_TXEN, CFG, SETUP_TIME, reg_cfg);
    txen->cr_xgwan_top_wan_misc_early_txen_cfg_toff_time = RU_FIELD_GET(0, EARLY_TXEN, CFG, TOFF_TIME, reg_cfg);
    txen->cr_xgwan_top_wan_misc_early_txen_cfg_output_txen_polarity = RU_FIELD_GET(0, EARLY_TXEN, CFG, OUTPUT_TXEN_POLARITY, reg_cfg);
    txen->cr_xgwan_top_wan_misc_early_txen_cfg_input_txen_polarity = RU_FIELD_GET(0, EARLY_TXEN, CFG, INPUT_TXEN_POLARITY, reg_cfg);
    txen->cr_xgwan_top_wan_misc_early_txen_cfg_early_txen_bypass = RU_FIELD_GET(0, EARLY_TXEN, CFG, EARLY_TXEN_BYPASS, reg_cfg);

    return BDMF_ERR_OK;
}

